/**
 * ドキュメント要素ユーティリティクラス<br>
 * <br>
 * ドキュメント要素に対する操作インタフェースを提供します。<br>
 *
 * @author Kitagawa<br>
 *
 *<!--
 * 更新日		更新者				更新内容
 * 2012/07/02	Kitagawa			新規作成
 * 2013/08/02	Kitagawa			getWidthPx、getHeightPx追加
 *-->
 */

/**
 * コンストラクタ<br>
 */
var ElementUtil = function() {
};

/**
 * 指定されたem単位サイズをpx単位に変換します。<br>
 * @param {Number} value em単位サイズ
 * @returns {Number} px単位サイズ 
 */
ElementUtil.convertEmToPx = function(value) {
	var fontSize = parseInt($("body").css("font-size"));
	return value * fontSize;
};

/**
 * 指定されたpx単位サイズをex単位に変換します。<br>
 * @param {Number} value px単位サイズ
 * @returns {Number} em単位サイズ 
 */
ElementUtil.convertPxToEm = function(value) {
	var fontSize = parseInt($("body").css("font-size"));
	return value / fontSize;
};

/**
 * 指定された要素のzIndex値を取得します。<br>
 * @param {Object} target 対象要素又はセレクタ文字列(未指定時はbody要素として実行されます)
 * @returns {Number} zIndex値
 */
ElementUtil.getZIndex = function(target) {
	// 対象要素取得
	target = !target ? "body" : target;

	var elementZIndex = 0;
	var element = $(target)[0];
	if (element.style) {
		elementZIndex = element.style.zIndex;
		if (!elementZIndex) {
			var css = element.currentStyle || document.defaultView.getComputedStyle(element, null);
			elementZIndex = css ? css.zIndex : 0;
		}
		elementZIndex -= 0;
		if (isNaN(elementZIndex)) {
			elementZIndex = 0;
		}
	}

	return elementZIndex;
};

/**
 * フォーカス位置の初期化を行います。<br>
 * input、select、textareaのうち、フォーカス可能な最初の要素にフォーカスを設定します。<br>
 * @param baseJElement 基底要素(JQueryオブジェクト)
 * @returns 正常終了した場合にtrueを返却
 */
ElementUtil.initFocus = function() {
	var focused = false;
	var $target = void 0;
	if (!$target) {
		$target = $("body");
	}
	$target.find("input:not([tabindex=-1][type=hidden]):visible:enabled,select:not([tabindex=-1]):visible:enabled,textarea:not([tabindex=-1]):visible:enabled").each(function(index, element) {
		if (focused) {
			return true;
		}
		$(element).focus();
		focused = true;
		return true;
	});
	return true;
};

/**
 * 画面上に未追加のjQueryオブジェクトについてもピクセル幅を取得します。<br>
 * @param {Object} target 対象要素又はセレクタ文字列又は、サイズ指定文字列
 * @returns 正常終了した場合にtrueを返却
 */
ElementUtil.getWidthPx = function(target) {
	if (typeof (target) === "undefined") {
		return 0;
	} else if (target.toString().endsWith("%") //
			|| target.toString().endsWith("in") //
			|| target.toString().endsWith("cm") //
			|| target.toString().endsWith("mm") //
			|| target.toString().endsWith("pt") //
			|| target.toString().endsWith("pc") //
			|| target.toString().endsWith("px") //
			|| target.toString().endsWith("em") //
			|| target.toString().endsWith("ex") //
			|| StringUtil.isNumeric(target.toString()) //
	) {
		var $dummy = $("<div/>");
		$dummy.css("display", "none");
		$dummy.width(target);
		return ElementUtil.getWidthPx($dummy);
	} else {
		var $target = $(target).clone();
		var display = $target.css("display");
		$target.css("display", "none");
		$("body").append($target);
		var width = $target.width();
		$target.remove();
		$target.css("display", display);
		return width;
	}
};

/**
 * 画面上に未追加のjQueryオブジェクトについてもピクセル高を取得します。<br>
 * @param {Object} target 対象要素又はセレクタ文字列又は、サイズ指定文字列
 * @returns 正常終了した場合にtrueを返却
 */
ElementUtil.getHeightPx = function(target) {
	if (typeof (target) === "undefined") {
		return 0;
	} else if (target.toString().endsWith("%") //
			|| target.toString().endsWith("in") //
			|| target.toString().endsWith("cm") //
			|| target.toString().endsWith("mm") //
			|| target.toString().endsWith("pt") //
			|| target.toString().endsWith("pc") //
			|| target.toString().endsWith("px") //
			|| target.toString().endsWith("em") //
			|| target.toString().endsWith("ex") //
			|| StringUtil.isNumeric(target.toString()) //
	) {
		var $dummy = $("<div/>");
		$dummy.css("display", "none");
		$dummy.height(target);
		return ElementUtil.getHeightPx($dummy);
	} else {
		var $target = $(target).clone();
		var display = $target.css("display");
		$target.css("display", "none");
		$("body").append($target);
		var height = $target.height();
		$target.remove();
		$target.css("display", display);
		return height;
	}
};
