package org.phosphoresce.lib.poi.dyna;

import org.phosphoresce.lib.poi.PoiGlobal;

/**
 * シートへの動的バインド値を管理するインタフェースを提供します。<br>
 * <p>
 * 動的設定を行うバインド値は全てこのクラスインスタンスとなります。<br>
 * 動的設定マップ情報での値タイプを統一化する為に{@link PoiDynaValue}を実装したバインド値情報クラスとして提供されています。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2012/07/09	Kitagawa		新規作成(バージョン2アップデート時に追加されました)
 *-->
 */
public class PoiDynaObject implements PoiGlobal, PoiDynaValue {

	/** バインドオブジェクト */
	private Object value;

	/**
	 * コンストラクタ<br>
	 * @param object バインド値
	 */
	public PoiDynaObject(Object object) {
		super();
		this.value = object;
	}

	/**
	 * コンストラクタ<br>
	 */
	public PoiDynaObject() {
		this(null);
	}

	/**
	 * バインド値タイプを取得します。<br>
	 * @return バインド値タイプ
	 * @see org.phosphoresce.lib.poi.dyna.PoiDynaValue#getDynaValueType()
	 */
	public int getDynaValueType() {
		return TYPE_OBJECT;
	}

	/**
	 * バインドオブジェクトを取得します。<br>
	 * @return バインドオブジェクト
	 */
	public Object getValue() {
		return value;
	}

	/**
	 * バインドオブジェクトを設定します。<br>
	 * @param value バインドオブジェクト
	 */
	public void setValue(Object value) {
		this.value = value;
	}

	/**
	 * クラス情報を文字列で取得します。<br>
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return value == null ? "null" : value.toString();
	}
}
