package sample.org.phosphoresce.webcore.ext.struts.common.action;

import org.phosphoresce.webcore.ext.hibernate4.dao.SCodeDAO;
import org.phosphoresce.webcore.ext.hibernate4.dao.SUserDAO;
import org.phosphoresce.webcore.ext.hibernate4.model.SUser;
import org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractHibernateService;
import org.phosphoresce.webcore.ext.hibernate4.transaction.HibernateTransactionManager;
import org.phosphoresce.webcore.ext.struts.annotation.StrutsService;

/**
 * サンプルアプリケーション共通上位抽象Hibernateサービスクラスです。<br>
 * <p>
 * Hibernateアドオンモジュールを利用したデータベースアクセスを行う業務処理実行サービス上位クラスです。<br>
 * アプリケーションで作成するサービスクラスは全てこのクラスを継承して作成します。<br>
 * </p>
 * <p>
 * 当クラスではHibernateを利用するサービスクラスにおける以下に示す共通のアノテーション付与及び、総称型の指定を行います。<br>
 * 機能ごとのサブサービスクラスではこのクラスで決定された共通のアノテーション、総称型指定は省略して実装を行います。<br>
 * <ul>
 * <li>サービスクラスアノテーション({@link StrutsService})</li>
 * トランザクションコンテキストクラスとして{@link SampleHibernateTransactionContext}、
 * トランザクション管理クラスとして{@link HibernateTransactionManager}、
 * トランザクション処理ユーザーとして{@link SUser}
 * を共通で提供します。<br>
 * のアノテーション定義を共通で提供します。<br>
 * <li>サービスクラス総称型</li>
 * {@link AbstractHibernateService}の総称型パラメータのうち、処理ユーザーに{@link SUser}を共通で決定します。<br>
 * </ul>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/07/11	Kitagawa		新規作成
 *-->
 */
@StrutsService( //
transactionContext = SampleHibernateTransactionContext.class, //
transactionManager = HibernateTransactionManager.class, //
transactionUser = SUser.class //
)
public abstract class SampleHibernateService extends AbstractHibernateService<SUser> {

	/** システムコードマスタテーブルアクセスオブジェクト */
	protected SCodeDAO sCodeDAO;

	/** システムユーザーテーブルアクセスオブジェクト */
	protected SUserDAO sUserDAO;

	/**
	 * コンストラクタ<br>
	 * @param transactionContext トランザクション処理コンテキストオブジェクト
	 */
	public SampleHibernateService(SampleHibernateTransactionContext transactionContext) {
		super(transactionContext);
		sCodeDAO = createDAO(SCodeDAO.class);
		sUserDAO = createDAO(SUserDAO.class);
	}
}
