package sample.org.phosphoresce.webcore.ext.struts.common.service;

import org.phosphoresce.lib.commons.util.StringUtil;
import org.phosphoresce.webcore.ext.hibernate4.model.SUser;
import org.phosphoresce.webcore.ext.hibernate4.util.SCodeManager;

import sample.org.phosphoresce.webcore.ext.struts.common.action.SampleHibernateService;
import sample.org.phosphoresce.webcore.ext.struts.common.action.SampleHibernateTransactionContext;

/**
 * ユーザーマスタテーブルサンプルサービスクラスです。<br>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/07/11	Kitagawa		新規作成
 *-->
 */
public class SampleSUserService extends SampleHibernateService {

	/**
	 * コンストラクタ<br>
	 * @param transactionContext トランザクション処理コンテキストオブジェクト
	 */
	public SampleSUserService(SampleHibernateTransactionContext transactionContext) {
		super(transactionContext);
	}

	/**
	 * 指定されたユーザーコードとパスワードで一致するユーザー情報を取得します。<br>
	 * パスワード一致比較は平文による比較及び、MD5ハッシュによる比較が行われ、いずれかで一致した場合に合致ユーザであると判断します。<br>
	 * @param code ユーザーコード
	 * @param password パスワード
	 * @return 合致するユーザーが存在した場合にそのエンティティを返却します
	 */
	public SUser searchByCodeAndPassword(String code, String password) {
		SUser sUser = sUserDAO.findByCode(code);
		if (sUser != null) {
			if (StringUtil.equals(password, sUser.getPassword())) {
				return sUser;
			} else if (StringUtil.equals(StringUtil.toMD5Hash(password), sUser.getPassword())) {
				return sUser;
			}
		}

		return null;
	}

	public long countAllUser() {
		return sUserDAO.countAll();
	}

	/**
	 * 指定された内容で新規にユーザーを登録します。<br>
	 * @param code ユーザーID
	 * @param password パスワード
	 * @param firstName 氏名(名)
	 * @param lastName 氏名(姓)
	 * @return 登録されたユーザーエンティティ
	 */
	public SUser createNewUser(String code, String password, String firstName, String lastName) {
		SUser sUser = new SUser();
		sUser.setCode(code);
		sUser.setPassword(StringUtil.toMD5Hash(password));
		sUser.setFirstName(firstName);
		sUser.setLastName(lastName);
		sUser.setRoleType(SCodeManager.getCode("S0002", "0"));

		save(sUser);

		return sUser;
	}
}
