package sample.org.phosphoresce.webcore.ext.struts.function.FS00001;

import org.phosphoresce.webcore.ext.hibernate4.model.SUser;
import org.phosphoresce.webcore.ext.struts.action.Forward;
import org.phosphoresce.webcore.ext.struts.action.LoginUser;
import org.phosphoresce.webcore.ext.struts.action.ServletContainer;
import org.phosphoresce.webcore.ext.struts.action.WindowSessionContainer;
import org.phosphoresce.webcore.ext.struts.annotation.StrutsMethod;

import sample.org.phosphoresce.webcore.ext.struts.common.action.SampleAction;

/**
 * サンプル機能アクションクラスです(ログイン画面)。<br>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2010/07/03	Kitagawa		新規作成
 *-->
 */
public class FS00001Action extends SampleAction<FS00001Form> implements FS00001Constants {

	/**
	 * 初期表示処理を行います。<br>
	 * @param form ActionFormオブジェクト
	 * @param session セッションコンテナオブジェクト
	 * @param servlet サーブレットコンテナオブジェクト
	 * @return 遷移先パス
	 * @throws Throwable 処理中に補足されない予期せぬエラーが発生した場合にスローされます
	 * @see org.phosphoresce.webcore.ext.struts.action.AbstractAction#index(org.phosphoresce.webcore.ext.struts.action.AbstractForm, org.phosphoresce.webcore.ext.struts.action.WindowSessionContainer, org.phosphoresce.webcore.ext.struts.action.ServletContainer)
	 */
	@Override
	@StrutsMethod(checkSesion = false, checkLogin = false)
	public Forward index(FS00001Form form, WindowSessionContainer<SUser> session, ServletContainer<SUser> servlet) throws Throwable {
		return new Forward();
	}

	/**
	 * ログインボタン押下時の処理を行います。<br>
	 * @param form ActionFormオブジェクト
	 * @param session セッションコンテナオブジェクト
	 * @param servlet サーブレットコンテナオブジェクト
	 * @return 遷移先パス
	 * @throws Throwable 処理中に補足されない予期せぬエラーが発生した場合にスローされます
	 * @see org.phosphoresce.webcore.ext.struts.action.AbstractAction#index(org.phosphoresce.webcore.ext.struts.action.AbstractForm, org.phosphoresce.webcore.ext.struts.action.WindowSessionContainer, org.phosphoresce.webcore.ext.struts.action.ServletContainer)
	 */
	@StrutsMethod(checkSesion = false, checkLogin = false, transaction = true, validate = "validateLogin")
	public Forward login(FS00001Form form, WindowSessionContainer<SUser> session, ServletContainer<SUser> servlet) throws Throwable {
		FS00001Logic fs00001Logic = createLogic(servlet, FS00001Logic.class);

		/*
		 * ユーザー情報が1件も存在しない場合は管理者ユーザーを作成
		 */
		fs00001Logic.createAdministrator();

		/*
		 * ログインユーザー情報の取得
		 */
		SUser sUser = fs00001Logic.searchLoginUser(form);
		if (sUser == null) {
			addMessage(servlet, "AMSG00001");
			return new Forward();
		}

		/*
		 * ログインユーザー情報のセッション設定
		 */
		LoginUser<SUser> loginUser = new LoginUser<SUser>();
		loginUser.setLoginDate(servlet.getProcessDate());
		loginUser.setUserCode(sUser.getCode());
		loginUser.setLoginInfo(sUser);
		session.setLoginUser(loginUser);

		return new Forward("/FS00002");
	}
}
