package sample.org.phosphoresce.webcore.ext.struts.function.FS00002;

import java.util.LinkedList;
import java.util.List;

import org.apache.struts.util.LabelValueBean;
import org.phosphoresce.lib.commons.util.StringUtil;
import org.phosphoresce.webcore.ext.hibernate4.model.SUser;
import org.phosphoresce.webcore.ext.struts.action.ServletContainer;
import org.phosphoresce.webcore.ext.struts.action.WindowSessionContainer;
import org.phosphoresce.webcore.ext.struts.annotation.CacheField;

import sample.org.phosphoresce.webcore.ext.struts.common.action.SampleForm;

/**
 * サンプル機能フォームクラスです(メニュー画面)。<br>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2010/07/03	Kitagawa		新規作成
 *-->
 */
public class FS00002Form extends SampleForm implements FS00002Constants {

	/** 選択済みタブインデックス */
	private String selectedTabIndex;

	/** 単項目バリデーション試験フィールド */
	private String fieldTestValidation;

	/** 編集監視試験フィールド */
	private String fieldTestMonitorChange;

	/** サーバー接続プリンター名リスト */
	@CacheField
	private List<String> printerNameList;

	/** 直接印刷プリンター名 */
	private String printerName;

	/** 非同期試験サンプルリスト */
	@CacheField
	private List<LabelValueBean> asyncSampleList;

	/** 非同期試験サンプルコード */
	private String asyncSample;

	/** 非同期試験サンプルサブリスト */
	@CacheField
	private List<LabelValueBean> asyncSampleSubList;

	/** 非同期試験サンプルサブコード */
	private String asyncSampleSub;

	/** ダイアログ連携試験フィールド */
	private String fieldTestDialog;

	/** Hibernate試験ユーザーコード */
	@CacheField
	private String fieldHibernateUserCd;

	/** Hibernate試験ユーザー名 */
	@CacheField
	private String fieldHibernateUserName;

	/**
	 * フォームオブジェクトをリセットします。<br>
	 * @param servlet サーブレットコンテナオブジェクト
	 * @param session ウィンドウセッションオブジェクト
	 * @see org.phosphoresce.webcore.ext.struts.action.AbstractForm#reset(org.phosphoresce.webcore.ext.struts.action.ServletContainer, org.phosphoresce.webcore.ext.struts.action.WindowSessionContainer)
	 */
	@Override
	public void reset(ServletContainer<SUser> servlet, WindowSessionContainer<SUser> session) {
	}

	/**
	 * フォームオブジェクトを初期化します。<br>
	 * @param servlet サーブレットコンテナオブジェクト
	 * @param session ウィンドウセッションオブジェクト
	 * @see org.phosphoresce.webcore.ext.struts.action.AbstractForm#init(org.phosphoresce.webcore.ext.struts.action.ServletContainer, org.phosphoresce.webcore.ext.struts.action.WindowSessionContainer)
	 */
	@Override
	public void init(ServletContainer<SUser> servlet, WindowSessionContainer<SUser> session) {
		fieldTestMonitorChange = "";
		fieldTestValidation = "";
		fieldTestMonitorChange = "";
		printerNameList = new LinkedList<String>();
		printerName = "";
		asyncSampleList = new LinkedList<LabelValueBean>();
		asyncSample = "";
		asyncSampleSubList = new LinkedList<LabelValueBean>();
		asyncSampleSub = "";
		fieldTestDialog = "";
		fieldHibernateUserCd = "";
		fieldHibernateUserName = "";
	}

	/**
	 * 項目バリデーションの実行ボタン押下時のバリデーションチェックを行います。<br>
	 * @param session ウィンドウセッションオブジェクト
	 * @return バリデーションエラーとなった場合にfalseを返却
	 */
	public boolean validateTestValidation(WindowSessionContainer<SUser> session) {
		boolean result = true;

		// テストフィールドが空の場合はエラー
		if (StringUtil.isEmpty(fieldTestValidation)) {
			addMessage("FCRV10001", "テストフィールド");
			result = false;
		}

		return result;
	}

	/**
	 * 選択済みタブインデックスを取得します。<br>
	 * @return 選択済みタブインデックス
	 */
	public String getSelectedTabIndex() {
		return selectedTabIndex;
	}

	/**
	 * 選択済みタブインデックスを設定します。<br>
	 * @param selectedTabIndex 選択済みタブインデックス
	 */
	public void setSelectedTabIndex(String selectedTabIndex) {
		this.selectedTabIndex = selectedTabIndex;
	}

	/**
	 * 単項目バリデーション試験フィールドを取得します。<br>
	 * @return 単項目バリデーション試験フィールド
	 */
	public String getFieldTestValidation() {
		return fieldTestValidation;
	}

	/**
	 * 単項目バリデーション試験フィールドを設定します。<br>
	 * @param fieldTestValidation 単項目バリデーション試験フィールド
	 */
	public void setFieldTestValidation(String fieldTestValidation) {
		this.fieldTestValidation = fieldTestValidation;
	}

	/**
	 * 編集監視試験フィールドを取得します。<br>
	 * @return 編集監視試験フィールド
	 */
	public String getFieldTestMonitorChange() {
		return fieldTestMonitorChange;
	}

	/**
	 * 編集監視試験フィールドを設定します。<br>
	 * @param fieldTestMonitorChange 編集監視試験フィールド
	 */
	public void setFieldTestMonitorChange(String fieldTestMonitorChange) {
		this.fieldTestMonitorChange = fieldTestMonitorChange;
	}

	/**
	 * サーバー接続プリンター名リストを取得します。<br>
	 * @return サーバー接続プリンター名リスト
	 */
	public List<String> getPrinterNameList() {
		return printerNameList;
	}

	/**
	 * サーバー接続プリンター名リストを設定します。<br>
	 * @param printerNameList サーバー接続プリンター名リスト
	 */
	public void setPrinterNameList(List<String> printerNameList) {
		this.printerNameList = printerNameList;
	}

	/**
	 * 直接印刷プリンター名を取得します。<br>
	 * @return 直接印刷プリンター名
	 */
	public String getPrinterName() {
		return printerName;
	}

	/**
	 * 直接印刷プリンター名を設定します。<br>
	 * @param printerName 直接印刷プリンター名
	 */
	public void setPrinterName(String printerName) {
		this.printerName = printerName;
	}

	/**
	 * 非同期試験サンプルリストを取得します。<br>
	 * @return 非同期試験サンプルリスト
	 */
	public List<LabelValueBean> getAsyncSampleList() {
		return asyncSampleList;
	}

	/**
	 * 非同期試験サンプルリストを設定します。<br>
	 * @param asyncSampleList 非同期試験サンプルリスト
	 */
	public void setAsyncSampleList(List<LabelValueBean> asyncSampleList) {
		this.asyncSampleList = asyncSampleList;
	}

	/**
	 * 非同期試験サンプルコードを取得します。<br>
	 * @return 非同期試験サンプルコード
	 */
	public String getAsyncSample() {
		return asyncSample;
	}

	/**
	 * 非同期試験サンプルコードを設定します。<br>
	 * @param asyncSample 非同期試験サンプルコード
	 */
	public void setAsyncSample(String asyncSample) {
		this.asyncSample = asyncSample;
	}

	/**
	 * 非同期試験サンプルサブリストを取得します。<br>
	 * @return 非同期試験サンプルサブリスト
	 */
	public List<LabelValueBean> getAsyncSampleSubList() {
		return asyncSampleSubList;
	}

	/**
	 * 非同期試験サンプルサブリストを設定します。<br>
	 * @param asyncSampleSubList 非同期試験サンプルサブリスト
	 */
	public void setAsyncSampleSubList(List<LabelValueBean> asyncSampleSubList) {
		this.asyncSampleSubList = asyncSampleSubList;
	}

	/**
	 * 非同期試験サンプルサブコードを取得します。<br>
	 * @return 非同期試験サンプルサブコード
	 */
	public String getAsyncSampleSub() {
		return asyncSampleSub;
	}

	/**
	 * 非同期試験サンプルサブコードを設定します。<br>
	 * @param asyncSampleSub 非同期試験サンプルサブコード
	 */
	public void setAsyncSampleSub(String asyncSampleSub) {
		this.asyncSampleSub = asyncSampleSub;
	}

	/**
	 * ダイアログ連携試験フィールドを取得します。<br>
	 * @return ダイアログ連携試験フィールド
	 */
	public String getFieldTestDialog() {
		return fieldTestDialog;
	}

	/**
	 * ダイアログ連携試験フィールドを設定します。<br>
	 * @param fieldTestDialog ダイアログ連携試験フィールド
	 */
	public void setFieldTestDialog(String fieldTestDialog) {
		this.fieldTestDialog = fieldTestDialog;
	}

	/**
	 * Hibernate試験ユーザーコードを取得します。<br>
	 * @return Hibernate試験ユーザーコード
	 */
	public String getFieldHibernateUserCd() {
		return fieldHibernateUserCd;
	}

	/**
	 * Hibernate試験ユーザーコードを設定します。<br>
	 * @param fieldHibernateUserCd Hibernate試験ユーザーコード
	 */
	public void setFieldHibernateUserCd(String fieldHibernateUserCd) {
		this.fieldHibernateUserCd = fieldHibernateUserCd;
	}

	/**
	 * Hibernate試験ユーザー名を取得します。<br>
	 * @return Hibernate試験ユーザー名
	 */
	public String getFieldHibernateUserName() {
		return fieldHibernateUserName;
	}

	/**
	 * Hibernate試験ユーザー名を設定します。<br>
	 * @param fieldHibernateUserName Hibernate試験ユーザー名
	 */
	public void setFieldHibernateUserName(String fieldHibernateUserName) {
		this.fieldHibernateUserName = fieldHibernateUserName;
	}
}
