package sample.org.phosphoresce.webcore.ext.struts.function.FS00002;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.phosphoresce.lib.poi.dyna.PoiDynaList;
import org.phosphoresce.lib.poi.dyna.PoiDynaMap;
import org.phosphoresce.lib.poi.dyna.PoiDynaObject;
import org.phosphoresce.webcore.ext.jasper.export.JasperContainer;

import sample.org.phosphoresce.webcore.ext.struts.common.action.SampleUtil;

/**
 * サンプル機能ユーティリティクラスです(メニュー画面)。<br>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2010/07/03	Kitagawa		新規作成
 *-->
 */
public class FS00002Util extends SampleUtil implements FS00002Constants {

	/**
	 * サンプル用のJasperReports出力用コンテナを生成します。<br>
	 * @return JasperReports出力用コンテナ
	 */
	public static JasperContainer createJasperSampleContainer() {
		JasperContainer container = new JasperContainer();

		// データソースレコード設定
		final int SECTION_COUNT = 3;
		//final int SECTION_EMPLOYEE_COUNT = 25000;
		final int SECTION_EMPLOYEE_COUNT = 100;
		final String[] SECTION_CODES = new String[] { "001", "002", "003" };
		final String[] SECTION_NAMES = new String[] { "○○システム部", "××システム部", "□□システム部" };
		for (int s = 0; s <= SECTION_COUNT - 1; s++) {
			for (int e = 0; e <= SECTION_EMPLOYEE_COUNT - 1; e++) {
				Map<String, Object> record = new HashMap<String, Object>();
				record.put("section_cd", SECTION_CODES[s]);
				record.put("section_name", SECTION_NAMES[s]);
				record.put("account_cd", SECTION_CODES[s] + "-" + e);
				record.put("last_name", "アイネス");
				record.put("first_name", "太郎");
				record.put("email", "test@test.com");
				record.put("create_date", "2012/07/01");
				record.put("modify_date", "2012/07/09");
				record.put("money", -200000);
				container.addRecord(record);
			}
		}

		// パラメータ設定
		container.setParameter("count", container.getRecordCount());

		return container;
	}

	/**
	 * POI帳票テスト用サンプルレコード(企業情報)を生成します。<br>
	 * @return レコードオブジェクト
	 */
	public static PoiDynaMap createSamplePOICompanyRecord() {
		PoiDynaMap map = new PoiDynaMap();
		map.put("nameJp", new PoiDynaObject("株式会社○○○○○"));
		map.put("nameEn", new PoiDynaObject("XXXXXX Corporation"));
		map.put("establishmentDate", new PoiDynaObject(new Date()));
		map.put("capital", new PoiDynaObject(1000000000L));
		map.put("employeeCount", new PoiDynaObject(1200));
		map.put("fiscalTerm", new PoiDynaObject("3月"));
		map.put("stockListing", new PoiDynaObject("2000年4月　東証第一部（証券コード　XXXX）"));
		map.put("address", new PoiDynaObject("東京都○○区×× 9-9-9\n□□ビル 2F"));
		return map;
	}

	/**
	 * POI帳票テスト用サンプルレコード(役員)を生成します。<br>
	 * @param type 役職タイプ
	 * @param name 役員名
	 * @return レコードオブジェクト
	 */
	public static PoiDynaMap createSamplePOIOfficerRecord(String type, String name) {
		PoiDynaMap map = new PoiDynaMap();
		map.put("type", new PoiDynaObject(type));
		map.put("name", new PoiDynaObject(System.currentTimeMillis() / 100000));
		return map;
	}

	/**
	 * POI帳票テスト用サンプルレコード(役員リスト)を生成します。<br>
	 * @return レコードオブジェクト
	 */
	public static PoiDynaList createSamplePOIOfficerList() {
		PoiDynaList list = new PoiDynaList();
		list.add(createSamplePOIOfficerRecord("取締役会長", "○○太郎"));
		list.add(createSamplePOIOfficerRecord("取締役", "○○二郎"));
		list.add(createSamplePOIOfficerRecord("取締役", "○○三郎"));
		return list;
	}

	/**
	 * POI帳票テスト用サンプルレコード(役員リストマップ)を生成します。<br>
	 * @param type 役職タイプ
	 * @return レコードオブジェクト
	 */
	public static PoiDynaMap createSamplePOIOfficerTypeListRecord(String type) {
		PoiDynaMap map = new PoiDynaMap();
		map.put("type", new PoiDynaObject(type));
		PoiDynaList list = new PoiDynaList();
		list.add(createSamplePOIOfficerRecord(type, "○○太郎"));
		list.add(createSamplePOIOfficerRecord(type, "○○二郎"));
		list.add(createSamplePOIOfficerRecord(type, "○○三郎"));
		map.put("officers", list);
		return map;
	}

	/**
	 * POI帳票テスト用サンプルレコード(役員リスト)を生成します。<br>
	 * @return レコードオブジェクト
	 */
	public static PoiDynaList createSamplePOIOfficerTypeList() {
		PoiDynaList list = new PoiDynaList();
		list.add(createSamplePOIOfficerTypeListRecord("取締役"));
		list.add(createSamplePOIOfficerTypeListRecord("監査役"));
		list.add(createSamplePOIOfficerTypeListRecord("執行役員"));
		return list;
	}
}
