package org.phosphoresce.webcore.core;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.NoSuchElementException;

import org.phosphoresce.lib.commons.util.StringUtil;

/**
 * アプリケーション環境設定の定義キーを提供します。<br>
 * <p>
 * 環境定義キーはリファクタリングによる容易な定義キー変更を可能とするため、文字列定数として提供されず、タイプセーフな列挙型クラスとして提供されます。<br>
 * また、サブモジュール環境定義名列挙クラスは必ずこれを継承して作成し、当クラスのコンストラクタを呼び出すサブクラスが設けられます。<br>
 * </p>
 * <p>
 * クラスの構成概要はorg.phosphoresce.webcore.coreパッケージAPIを参照してください。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2010/07/03	Kitagawa		新規作成
 * 2013/07/04	Kitagawa		定数インタフェースから列挙形式のクラスに変更(Configクラスアクセス時に必ず列挙クラス定義を行う制限を設けさせる為)
 *-->
 */
public class GlobalConfigName implements Serializable {

	/** アプリケーションタイトル定義 */
	public static final GlobalConfigName COMMON_APPLICATION_TITLE = new GlobalConfigName("common.application.title");

	/** アプリケーションタイトルイメージURLパス定義 */
	public static final GlobalConfigName COMMON_APPLICATION_TITLE_IMAGE_URL = new GlobalConfigName("common.application.title.image.url");

	/** アプリケーションフッター定義 */
	public static final GlobalConfigName COMMON_APPLICATION_FOOTER = new GlobalConfigName("common.application.footer");

	/** iOSモバイルSafari利用時のブックマーク強制誘導定義 */
	public static final GlobalConfigName COMMON_APPLICATION_IOS_BOOKMARK = new GlobalConfigName("common.application.ios.bookmark");

	/** ディフォルトキャラクタセット定義 */
	public static final GlobalConfigName COMMON_CHARSET_DEFAULT = new GlobalConfigName("common.charset.default");

	/** コンテンツ絶対パス定義 */
	public static final GlobalConfigName COMMON_ABSOLUTE_CONTENT_PATH = new GlobalConfigName("common.absolute.content.path");

	/** WEB-INF絶対パス定義 */
	public static final GlobalConfigName COMMON_ABSOLUTE_WEBINF_PATH = new GlobalConfigName("common.absolute.webinf.path");

	/** Javascript用サーバーメッセージ利用サーブレットパス定義 */
	public static final GlobalConfigName COMMON_SERVLET_PATH_JS_MESSAGE = new GlobalConfigName("common.webxml.jsmsg.url");

	/** サーバー静的リソースリクエストサーブレットパス定義 */
	public static final GlobalConfigName COMMON_SERVLET_PATH_STATIC_RESOURCE = new GlobalConfigName("common.webxml.staticres.url");

	/** 拡張初期化クラス定義 */
	public static final GlobalConfigName COMMON_INITIALIZE_CLASSES = new GlobalConfigName("common.initialize.classes");

	/** 追加読み込みプロパティ定義 */
	public static final GlobalConfigName COMMON_RESOURCE_CONFIG_EXTENDS = new GlobalConfigName("common.resource.config.extends");

	/** 文字列定義リソースパス定義 */
	public static final GlobalConfigName COMMON_RESOURCE_STRING = new GlobalConfigName("common.resource.string");

	/** MIMEタイプ対応リソース定義 */
	public static final GlobalConfigName COMMON_RESROUCE_MIME_TYPE = new GlobalConfigName("common.resource.mimetype");

	/** Logback環境設定リソースパス定義 */
	public static final GlobalConfigName COMMON_RESOURCE_LOGBACK = new GlobalConfigName("common.resource.logback.config");

	/** 汎用サーバダウンロードリソースパス定義 */
	public static final GlobalConfigName COMMON_GENERAL_DOWNLOAD_RESOURCE = new GlobalConfigName("common.resource.download.general");

	/** 汎用サーバアップロードリソースパス定義 */
	public static final GlobalConfigName COMMON_GENERAL_UPLOAD_RESOURCE = new GlobalConfigName("common.resource.upload.general");

	/** 汎用サーバテンポラリリソースパス定義 */
	public static final GlobalConfigName COMMON_GENERAL_TEMPORARY_RESOURCE = new GlobalConfigName("common.resource.temporary.general");

	/** 静的サーバリソースパス定義 */
	public static final GlobalConfigName COMMON_STATIC_RESOURCE = new GlobalConfigName("common.resource.static");

	/** メッセージ出力時メッセージコード出力フラグ */
	public static final GlobalConfigName COMMON_MESSAGE_SHOWCODE = new GlobalConfigName("common.message.showCode");

	/** アプリケーションガベージコレクションデーモン定義 */
	public static final GlobalConfigName COMMON_DAEMON_GC = new GlobalConfigName("common.daemon.gc");

	/** アプリケーションガベージコレクションデーモン実行間隔(ミリ秒定義 */
	public static final GlobalConfigName COMMON_DAEMON_GC_INTERVAL = new GlobalConfigName("common.daemon.gc.interval");

	/** Cookie有効時間(秒 */
	public static final GlobalConfigName COMMON_SERVLET_COOKIE_TIMEOUT = new GlobalConfigName("common.servlet.cookie.timeout");

	/** サーブレットページ配置基底パス定義 */
	public static final GlobalConfigName COMMON_JSP_PATH = new GlobalConfigName("common.jsp.path");

	/** システムエラーページパス定義 */
	public static final GlobalConfigName COMMON_JSP_SYSTEMERROR = new GlobalConfigName("common.jsp.systemerror");

	/** メッセージページパス定義 */
	public static final GlobalConfigName COMMON_JSP_MESSAGE = new GlobalConfigName("common.jsp.message");

	/** ウェルカムページパス定義 */
	public static final GlobalConfigName COMMON_JSP_WELCOME = new GlobalConfigName("common.jsp.welcome");

	/** ポップアップコンテナページパス定義 */
	public static final GlobalConfigName COMMON_JSP_POPUP = new GlobalConfigName("common.jsp.popup");

	/** ブックマーク誘導ページパス定義 */
	public static final GlobalConfigName COMMON_JSP_BOOKMARKING = new GlobalConfigName("common.jsp.bookmarking");

	/** プロパティキー */
	private String key;

	/** インスタンスキャッシュ */
	private static Map<String, GlobalConfigName> instances;

	/**
	 * コンストラクタ<br>
	 * @param key プロパティキー
	 */
	protected GlobalConfigName(String key) {
		if (key == null) {
			throw new NullPointerException();
		}
		this.key = key;
		if (instances == null) {
			instances = new HashMap<String, GlobalConfigName>();
		}
		synchronized (instances) {
			instances.put(key, this);
		}
	}

	/**
	 * 定義列挙型クラスインスタンスのプロパティキー文字列を取得します。<br>
	 * @return プロパティキー文字列
	 */
	public final String getKey() {
		return key;
	}

	/**
	 * 指定されたプロパティキー文字列の定義列挙型クラスインスタンスを取得します。<br>
	 * 存在しないプロパティキーが指定された場合は{@link NoSuchElementException}がスローされます。<br>
	 * @param key プロパティキー文字列
	 * @return 定義列挙型クラスインスタンス
	 * @throws NoSuchElementException 存在しないプロパティキーが指定された場合にスローされます
	 */
	public static final GlobalConfigName get(String key) throws NoSuchElementException {
		synchronized (instances) {
			if (instances == null || !instances.containsKey(key)) {
				throw new NoSuchElementException();
			}
			return instances.get(key);
		}
	}

	/**
	 * 自身の定義列挙型クラスインスタンスと同様の定義キーを持つ定義情報にサフィックスを装飾した定義インスタンスを提供します。<br>
	 * 例えば、"definition"という定義のインスタンスでサフィックスに"sub1"と指定した場合、"definition.sub1"という定義列挙型クラスインスタンスが提供されます。<br>
	 * @param suffix サフィックス文字列
	 * @return 定義列挙型クラスインスタンス
	 */
	public final GlobalConfigName suffixedName(String suffix) {
		if (StringUtil.isEmpty(suffix)) {
			return this;
		} else {
			String targetKey = key + suffix;
			synchronized (instances) {
				if (instances.containsKey(targetKey)) {
					return instances.get(targetKey);
				}
			}
			return new GlobalConfigName(key + suffix);
		}
	}

	/**
	 * クラス情報を文字列で定義します。<br>
	 * @return クラス情報文字列
	 * @see java.lang.Object#toString()
	 */
	@Override
	public final String toString() {
		return key;
	}
}
