package org.phosphoresce.webcore.core.exception;

import org.phosphoresce.webcore.core.GlobalConstants;

/**
 * フレームワークレベルで発生する全ての業務レベル例外の最上位に位置する抽象例外クラスです。<br>
 * <p>
 * フレームワーク内でスローされる業務例外はコアモジュールの例外を含めて全てこのクラスを継承したサブクラスで設計されます。<br>
 * このクラスを継承した例外クラスはすべてエラーメッセージまたは、業務エラーページに遷移してエラー内容をユーザーに示す動作しようとして例外制御が行われます。<br>
 * システムエラーとして扱うべき例外は{@link WebcoreCriticalException}を継承した例外クラスとしてクラスが設けられます。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2010/07/03	Kitagawa		新規作成
 *-->
 */
public abstract class WebcoreApplicationException extends WebcoreException implements GlobalConstants {

	/**
	 * コンストラクタ<br>
	 */
	public WebcoreApplicationException() {
		super();
	}

	/**
	 * コンストラクタ<br>
	 * @param code 例外メッセージコード
	 * @param cause 例外要因
	 */
	public WebcoreApplicationException(String code, Throwable cause) {
		super(code, cause);
	}

	/**
	 * コンストラクタ<br>
	 * @param code 例外メッセージコード
	 * @param binds 例外メッセージバインドオブジェクト
	 * @param cause 例外要因
	 */
	public WebcoreApplicationException(String code, Object[] binds, Throwable cause) {
		super(code, binds, cause);
	}

	/**
	 * コンストラクタ<br>
	 * @param code 例外メッセージ
	 */
	public WebcoreApplicationException(String code) {
		super(code);
	}

	/**
	 * コンストラクタ<br>
	 * @param code 例外メッセージ
	 * @param binds 例外メッセージバインドオブジェクト
	 */
	public WebcoreApplicationException(String code, Object[] binds) {
		super(code, binds);
	}
}
