package org.phosphoresce.webcore.core.transaction;

/**
 * トンランザクション処理を行うためのサービスクラスの上位抽象クラスとして提供されます。<br>
 * <p>
 * 当クラスはトランザクション処理コンテキストを基にトランザクション処理を行う処理集合体クラスの上位クラスとなります。<br>
 * 実際の具象化されるサービスクラスはトランザクションコンテキストクラスが提供する情報を元にトランザクション処理を行う暮らすとしてこれを継承して提供されます。<br>
 * </p>
 * 
 * @param <T> トランザクション管理クラス
 * @param <U> トランザクション処理ユーザークラス
 * @param <C> トランザクションコンテキストクラス
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2010/07/05	Kitagawa		新規作成
 *-->
 */
public abstract class TransactionService<T extends TransactionManager, U extends TransactionUser, C extends TransactionContext<T, U>> {

	/** トランザクション処理コンテキストオブジェクト */
	private C transactionContext;

	/**
	 * コンストラクタ<br>
	 * @param transactionContext トランザクション処理コンテキストオブジェクト
	 */
	public TransactionService(C transactionContext) {
		if (transactionContext == null) {
			throw new NullPointerException();
		}
		this.transactionContext = transactionContext;
	}

	/**
	 * トランザクション処理コンテキストオブジェクトを取得します。<br>
	 * @return トランザクション処理コンテキストオブジェクト
	 */
	protected final C getTransactionContext() {
		return transactionContext;
	}
}
