package org.phosphoresce.webcore.ext.hibernate4.dao;

import static org.hibernate.criterion.Restrictions.*;

import org.hibernate.SQLQuery;
import org.phosphoresce.webcore.ext.hibernate4.annotation.HibernateDAO;
import org.phosphoresce.webcore.ext.hibernate4.model.SCode;
import org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractGenericHibernateDAO;
import org.phosphoresce.webcore.ext.hibernate4.transaction.GenericCriteria;
import org.phosphoresce.webcore.ext.hibernate4.transaction.HibernateTransactionManager;

/**
 * システムコードマスタテーブルのエンティティへのアクセスインタフェースを提供します。<br>
 * <p>
 * テーブルエンティティに対する汎用的なアクセスインタフェースが提供されます。<br>
 * 業務処理依存するアクセスインタフェースは提供されません。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/02/01	Kitagawa		新規作成
 *-->
 */
@HibernateDAO(entity = SCode.class)
public class SCodeDAO extends AbstractGenericHibernateDAO<SCode> {

	/**
	 * コンストラクタ<br>
	 * @param transaction トランザクション管理オブジェクト
	 */
	public SCodeDAO(HibernateTransactionManager transaction) {
		super(transaction);
	}

	/**
	 * 指定されたカテゴリコード及び、コードでシステムコードマスタエンティティを検索します。<br>
	 * @param category カテゴリコード
	 * @param code コード
	 * @return システムコードマスタエンティティ
	 */
	public SCode findByCategoryAndCode(String category, String code) {
		GenericCriteria<SCode> criteria = createCriteria();
		criteria.add(eq("category", category));
		criteria.add(eq("code", code));
		return criteria.uniqueResult();
	}

	/**
	 * 全てのシステムコードマスタを物理削除します。<br>
	 */
	public void removeAll() {
		SQLQuery query = createSQLQuery("delete from s_code");
		query.executeUpdate();
	}
}
