package org.phosphoresce.webcore.ext.hibernate4.dao;

import static org.hibernate.criterion.Restrictions.*;

import org.phosphoresce.webcore.ext.hibernate4.annotation.HibernateDAO;
import org.phosphoresce.webcore.ext.hibernate4.model.SUser;
import org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractGenericHibernateDAO;
import org.phosphoresce.webcore.ext.hibernate4.transaction.GenericCriteria;
import org.phosphoresce.webcore.ext.hibernate4.transaction.HibernateTransactionManager;

/**
 * システムユーザーテーブルのエンティティへのアクセスインタフェースを提供します。<br>
 * <p>
 * テーブルエンティティに対する汎用的なアクセスインタフェースが提供されます。<br>
 * 業務処理依存するアクセスインタフェースは提供されません。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/02/01	Kitagawa		新規作成
 *-->
 */
@HibernateDAO(entity = SUser.class)
public class SUserDAO extends AbstractGenericHibernateDAO<SUser> {

	/**
	 * コンストラクタ<br>
	 * @param transaction トランザクション管理オブジェクト
	 */
	public SUserDAO(HibernateTransactionManager transaction) {
		super(transaction);
	}

	/**
	 * 指定されたユーザーコードで検索を行います。<br>
	 * @param code ユーザーコード
	 * @return エンティティオブジェクト
	 */
	public SUser findByCode(String code) {
		GenericCriteria<SUser> criteria = createCriteriaExcludeDeleted();

		criteria.add(eq("code", code));

		return criteria.uniqueResult();
	}
}
