package org.phosphoresce.webcore.ext.hibernate4.model;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractHibernateEntity;

/**
 * システム環境設定テーブルのエンティティインタフェースを提供します。<br>
 * <p>
 * システム内部で管理される各種設定情報が保存されるテーブルエンティティです。<br>
 * アプリケーション側で業務的に直接利用されることは想定されません。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/02/01	Kitagawa		新規作成		
 *-->
 */
@Entity
@Table(name = "s_config", //
uniqueConstraints = { //
@UniqueConstraint(columnNames = { "property" }) //
})
public class SConfig extends AbstractHibernateEntity {

	/** プロパティキー */
	@Column(name = "property", nullable = false, length = 32)
	private String property;

	/** 設定値1 */
	@Column(name = "value_1", length = 50)
	private String value1;

	/** 設定値2 */
	@Column(name = "value_2", length = 50)
	private String value2;

	/** 設定値3 */
	@Column(name = "value_3", length = 50)
	private String value3;

	/** 設定値4 */
	@Column(name = "value_4", length = 50)
	private String value4;

	/** 設定値5 */
	@Column(name = "value_5", length = 50)
	private String value5;

	/**
	 * データベース登録処理前のエンティティ内容の整合処理を実施します。<br>
	 * 当処理はエンティティ毎に処理が異なります。<br>
	 * @see org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractHibernateEntity#validateEntity()
	 */
	@Override
	protected void validateEntity() {
	}

	/**
	 * プロパティキーを取得します。<br>
	 * @return プロパティキー
	 */
	public String getProperty() {
		return property;
	}

	/**
	 * プロパティキーを設定します。<br>
	 * @param property プロパティキー
	 */
	public void setProperty(String property) {
		this.property = property;
	}

	/**
	 * 設定値1を取得します。<br>
	 * @return 設定値1
	 */
	public String getValue1() {
		return value1;
	}

	/**
	 * 設定値1を設定します。<br>
	 * @param value1 設定値1
	 */
	public void setValue1(String value1) {
		this.value1 = value1;
	}

	/**
	 * 設定値2を取得します。<br>
	 * @return 設定値2
	 */
	public String getValue2() {
		return value2;
	}

	/**
	 * 設定値2を設定します。<br>
	 * @param value2 設定値2
	 */
	public void setValue2(String value2) {
		this.value2 = value2;
	}

	/**
	 * 設定値3を取得します。<br>
	 * @return 設定値3
	 */
	public String getValue3() {
		return value3;
	}

	/**
	 * 設定値3を設定します。<br>
	 * @param value3 設定値3
	 */
	public void setValue3(String value3) {
		this.value3 = value3;
	}

	/**
	 * 設定値4を取得します。<br>
	 * @return 設定値4
	 */
	public String getValue4() {
		return value4;
	}

	/**
	 * 設定値4を設定します。<br>
	 * @param value4 設定値4
	 */
	public void setValue4(String value4) {
		this.value4 = value4;
	}

	/**
	 * 設定値5を取得します。<br>
	 * @return 設定値5
	 */
	public String getValue5() {
		return value5;
	}

	/**
	 * 設定値5を設定します。<br>
	 * @param value5 設定値5
	 */
	public void setValue5(String value5) {
		this.value5 = value5;
	}
}
