package org.phosphoresce.webcore.ext.hibernate4.model.usertype;

import org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractHibernateEntity;

/**
 * エンティティ削除状態のタイプ定義を提供します。<br>
 * <p>
 * {@link AbstractHibernateEntity}を継承して作成されたエンティティはすべて論理削除状態を保持するエンティティとなります。<br>
 * 論理削除状態を表すフラグカラムはここで提供するタイプ定義が設定されます。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/02/01	Kitagawa		新規作成
 * 2013/07/25	Kitagawa		Enumとして設けられていたクラスをユーザタイプクラスに変更
 *-->
 */
public class DeleteType extends AbstractUserType<DeleteType> {

	/** 未削除の有効なエンティティ */
	public static final DeleteType VALIDATE = new DeleteType("0");

	/** 論理削除状態のエンティティ */
	public static final DeleteType DELETED = new DeleteType("1");

	/**
	 * 全てのタイプを提供します。<br>
	 * @return 全てのタイプ
	 * @see org.phosphoresce.webcore.ext.hibernate4.model.usertype.AbstractUserType#allTypes()
	 */
	@Override
	public DeleteType[] allTypes() {
		return new DeleteType[] { //
		//
				VALIDATE, //
				DELETED, //
		};
	}

	/**
	 * コンストラクタ<br>
	 */
	public DeleteType() {
		super();
	}

	/**
	 * コンストラクタ<br>
	 * @param code タイプコード
	 */
	protected DeleteType(String code) {
		super(code);
	}
}
