package org.phosphoresce.webcore.ext.hibernate4.service;

import java.util.LinkedList;
import java.util.List;

import org.hibernate.criterion.Order;
import org.phosphoresce.webcore.ext.hibernate4.dao.SCodeDAO;
import org.phosphoresce.webcore.ext.hibernate4.dao.SConfigDAO;
import org.phosphoresce.webcore.ext.hibernate4.dao.SUserDAO;
import org.phosphoresce.webcore.ext.hibernate4.model.SCode;
import org.phosphoresce.webcore.ext.hibernate4.model.SConfig;
import org.phosphoresce.webcore.ext.hibernate4.model.SUser;
import org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractHibernateService;
import org.phosphoresce.webcore.ext.hibernate4.transaction.HibernateTransactionContext;

/**
 * システムマスタ処理用のサービスインタフェースを提供します。<br>
 * <p>
 * 当サービスクラスはHibernateアドオンモジュールとして共通提供されるサービスクラスです。<br>
 * システムコードマスタに対する操作を始め、システムマスタテーブルに関する各種処理が提供されます。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/07/31	Kitagawa		新規作成
 *-->
 */
public class SystemMasterService extends AbstractHibernateService<SUser> {

	/** システム環境設定マスタテーブルアクセスオブジェクト */
	private SConfigDAO sConfigDAO;

	/** システムコードマスタテーブルアクセスオブジェクト */
	private SCodeDAO sCodeDAO;

	/** システムユーザーテーブルアクセスオブジェクト */
	private SUserDAO sUserDAO;

	/**
	 * コンストラクタ<br>
	 * @param transactionContext トランザクション処理コンテキストオブジェクト
	 */
	public SystemMasterService(HibernateTransactionContext<SUser> transactionContext) {
		super(transactionContext);
		sConfigDAO = createDAO(SConfigDAO.class);
		sCodeDAO = createDAO(SCodeDAO.class);
		sUserDAO = createDAO(SUserDAO.class);
	}

	/**
	 * システムに登録されている全てのシステム環境設定マスタを取得します。<br>
	 * @return 全てのシステム環境設定マスタエンティティリスト
	 */
	public List<SConfig> getAllSConfig() {
		List<Order> orders = new LinkedList<Order>();
		orders.add(Order.asc("property"));
		return sConfigDAO.findAll(orders);
	}

	/**
	 * 指定されたプロパティのシステム環境設定マスタを取得します。<br>
	 * @return システム環境設定マスタエンティティ
	 */
	public SConfig getSConfigByProperty(String property) {
		return sConfigDAO.findByProperty(property);
	}

	/**
	 * 指定されたシステム環境設定マスタを登録します。<br>
	 * @param sCode システム環境設定マスタエンティティ
	 */
	public void registSConfig(SConfig sConfig) {
		save(sConfig);
	}

	/**
	 * システムに登録されている全てのシステムコードマスタを取得します。<br>
	 * @return 全てのシステムコードマスタエンティティリスト
	 */
	public List<SCode> getAllSCode() {
		List<Order> orders = new LinkedList<Order>();
		orders.add(Order.asc("category"));
		orders.add(Order.asc("orderNo"));
		return sCodeDAO.findAll(orders);
	}

	/**
	 * 指定されたシステムコードマスタを登録します。<br>
	 * @param sCode システムコードマスタエンティティ
	 */
	public void registSCode(SCode sCode) {
		save(sCode);
	}

	/**
	 * 全てのシステムコードマスタを物理削除します。<br>
	 */
	public void removeAllSCode() {
		sCodeDAO.removeAll();
	}
}
