package org.phosphoresce.webcore.ext.hibernate4.transaction;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.MappedSuperclass;

import org.hibernate.annotations.JoinColumnOrFormula;
import org.hibernate.annotations.JoinColumnsOrFormulas;
import org.hibernate.annotations.JoinFormula;
import org.hibernate.annotations.Where;
import org.phosphoresce.webcore.ext.hibernate4.model.SCode;
import org.phosphoresce.webcore.ext.hibernate4.model.SUser;

/**
 * 通常のアプリケーションデータベーステーブルエンティティの共通カラムフィールドインタフェースを提供します。<br>
 * <p>
 * Hibernateアドオンモジュールを利用したアプリケーションで利用する通常のエンティティはすべてこのクラスが継承されて提供されます。<br>
 * これはすべてのエンティティはレコードID、レコードバージョン番号及び、登録日時、登録者情報が必ず管理されることを意味します。<br>
 * また、これらのアクセッサインタフェースはユーザーからの利用ではGetterインタフェースのみ提供され、Setterインタフェースはアドオンモジュール側で制御するのみに制限されます。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/02/01	Kitagawa		新規作成
 *-->
 */
@MappedSuperclass
public abstract class AbstractHibernateDefaultEntity extends AbstractHibernateEntity {

	/** 登録日時 */
	@Column(name = "entry_date")
	private Date entryDate;

	/** 登録ユーザ */
	@ManyToOne
	@JoinColumn(name = "entry_user", referencedColumnName = "code")
	@Where(clause = "delete_type = '0'")
	private SUser entryUser;

	/** 更新日時 */
	@Column(name = "modify_date")
	private Date modifyDate;

	/** 更新ユーザ */
	@ManyToOne
	@JoinColumn(name = "modify_user", referencedColumnName = "code")
	@Where(clause = "delete_type = '0'")
	private SUser modifyUser;

	/** 削除日時 */
	@Column(name = "delete_date")
	private Date deleteDate;

	/** 削除ユーザ */
	@ManyToOne
	@JoinColumn(name = "delete_user", referencedColumnName = "code")
	@Where(clause = "delete_type = '0'")
	private SUser deleteUser;

	/** 削除状態 */
	@ManyToOne
	//@JoinFormula(referencedColumnName = "code", value = "(select s_code.code from s_code where s_code.category = 'S0001' and s_code.code = delete_type)")
	@JoinColumnsOrFormulas({ //
	@JoinColumnOrFormula(formula = @JoinFormula(referencedColumnName = "category", value = "'S0001'")), //
			@JoinColumnOrFormula(column = @JoinColumn(referencedColumnName = "code", name = "delete_type", nullable = false)) //
	})
	private SCode deleteType;

	/**
	 * データベース登録処理前のエンティティ内容の整合処理を実施します。<br>
	 */
	void validate() {
		validateEntity();
	}

	/**
	 * データベース登録処理前のエンティティ内容の整合処理を実施します。<br>
	 * 当処理はエンティティ毎に処理が異なります。<br>
	 */
	protected abstract void validateEntity();

	/**
	 * 登録日時を取得します。<br>
	 * @return 登録日時
	 */
	public final Date getEntryDate() {
		return entryDate;
	}

	/**
	 * 登録日時を設定します。<br>
	 * @param entryDate 登録日時
	 */
	final void setEntryDate(Date entryDate) {
		this.entryDate = entryDate;
	}

	/**
	 * 登録ユーザを取得します。<br>
	 * @return 登録ユーザ
	 */
	public SUser getEntryUser() {
		return entryUser;
	}

	/**
	 * 登録ユーザを設定します。<br>
	 * @param entryUser 登録ユーザ
	 */
	void setEntryUser(SUser entryUser) {
		this.entryUser = entryUser;
	}

	/**
	 * 更新日時を取得します。<br>
	 * @return 更新日時
	 */
	public final Date getModifyDate() {
		return modifyDate;
	}

	/**
	 * 更新日時を設定します。<br>
	 * @param modifyDate 更新日時
	 */
	final void setModifyDate(Date modifyDate) {
		this.modifyDate = modifyDate;
	}

	/**
	 * 更新ユーザを取得します。<br>
	 * @return 更新ユーザ
	 */
	public SUser getModifyUser() {
		return modifyUser;
	}

	/**
	 * 更新ユーザを設定します。<br>
	 * @param modifyUser 更新ユーザ
	 */
	void setModifyUser(SUser modifyUser) {
		this.modifyUser = modifyUser;
	}

	/**
	 * 削除日時を取得します。<br>
	 * @return 削除日時
	 */
	public final Date getDeleteDate() {
		return deleteDate;
	}

	/**
	 * 削除日時を設定します。<br>
	 * @param deleteDate 削除日時
	 */
	final void setDeleteDate(Date deleteDate) {
		this.deleteDate = deleteDate;
	}

	/**
	 * 削除ユーザを取得します。<br>
	 * @return 削除ユーザ
	 */
	public SUser getDeleteUser() {
		return deleteUser;
	}

	/**
	 * 削除ユーザを設定します。<br>
	 * @param deleteUser 削除ユーザ
	 */
	public void setDeleteUser(SUser deleteUser) {
		this.deleteUser = deleteUser;
	}

	/**
	 * 削除状態を取得します。<br>
	 * @return 削除状態
	 */
	public SCode getDeleteType() {
		return deleteType;
	}

	/**
	 * 削除状態を設定します。<br>
	 * @param deleteType 削除状態
	 */
	public void setDeleteType(SCode deleteType) {
		this.deleteType = deleteType;
	}
}
