package org.phosphoresce.webcore.ext.jasper.export;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.phosphoresce.lib.commons.util.StringUtil;

/**
 * JasperReportsバインドパラメータ設定時に環境依存文字による例外発生を防止する為のフィルタインタフェースを提供します。<br>
 * <p>
 * バインドパラメータ又は、データソースにおいて環境依存文字が存在する場合、iTextライブラリにおいて例外が発生します。<br>
 * バインドデータ登録時に入力チェック等で制限がかかっていない場合に当クラスを介すことで強制的に
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/08/13	Kitagawa		新規作成
 *-->
 */
public class JasperCharacterFilter {

	/** 環境依存代替文字 */
	public static String SUBSTITUDE_CHAR = "?";

	/**
	 * コンストラクタ<br>
	 */
	private JasperCharacterFilter() {
		super();
	}

	/**
	 * 指定された文字列に環境依存文字が含まれる場合、代替文字に置き換えて提供します。<br>
	 * @param value 対象文字列
	 * @return 編集後文字列
	 */
	public static String encode(String value) {
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i <= value.length() - 1; i++) {
			String test = new String(new char[] { value.charAt(i) });
			String etest = StringUtil.encode(test, "MS932");
			if ("?".equals(etest)) {
				buffer.append(SUBSTITUDE_CHAR);
			} else {
				buffer.append(value.charAt(i));
			}
		}
		return buffer.toString();
	}

	/**
	 * 指定されたマップ情報の値が文字列の場合で環境文字が含まれる場合、代替文字列に置き換えて提供します。<br>
	 * 指定されたマップインスタンス自体の編集は行わずに、新たなマップインスタンスとして提供されます。<br>
	 * @param data 対象マップオブジェクト
	 * @return 編集後マップオブジェクト
	 */
	public static Map<String, Object> encode(Map<String, Object> data) {
		if (data == null) {
			return null;
		}
		Map<String, Object> render = new HashMap<String, Object>();
		for (String key : data.keySet()) {
			Object value = data.get(key);
			if (value instanceof String) {
				render.put(key, encode((String) value));
			} else if (value instanceof String[]) {
				List<String> strings = new LinkedList<String>();
				for (String string : (String[]) value) {
					strings.add(encode(string));
				}
				render.put(key, strings.toArray(new String[0]));
			} else {
				render.put(key, value);
			}
		}
		return render;
	}

	/**
	 * 指定されたリスト情報の値が文字列の場合で環境文字が含まれる場合、代替文字列に置き換えて提供します。<br>
	 * 指定されたリストインスタンス自体の編集は行わずに、新たなマップインスタンスとして提供されます。<br>
	 * @param list 対象リストオブジェクト
	 * @return 編集後リストオブジェクト
	 */
	public static List<Map<String, Object>> encode(List<Map<String, Object>> list) {
		if (list == null) {
			return null;
		}
		List<Map<String, Object>> render = new LinkedList<Map<String, Object>>();
		for (Map<String, Object> data : list) {
			render.add(encode(data));
		}
		return render;
	}
}
