package org.phosphoresce.webcore.ext.jdbc;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

/**
 * アプリケーション実行中に永続的に保持されるJDBCアドオンモジュール情報を提供します。<br>
 * <p>
 * クラス概要はコアモジュール側パッケージドキュメントを参照してください。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2010/07/06	Kitagawa		新規作成
 *-->
 */
public class JDBCEnvironment implements Serializable, JDBCConstants {

	/** データベース接続可能フラグ */
	private static boolean enabledDatabase;

	/**
	 * 環境変数内容をMapオブジェクトとして取得します。<br>
	 * @return Mapオブジェクト
	 */
	public static Map<String, Object> getMap() {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("enabledDatabase", enabledDatabase);
		return map;
	}

	/**
	 * コンストラクタ<br>
	 */
	private JDBCEnvironment() {
		super();
	}

	/**
	 * データベース接続可能フラグを取得します。<br>
	 * @return データベース接続可能フラグ
	 */
	public static boolean isEnabledDatabase() {
		return enabledDatabase;
	}

	/**
	 * データベース接続可能フラグを設定します。<br>
	 * @param enabledDatabase データベース接続可能フラグ
	 */
	public static void setEnabledDatabase(boolean enabledDatabase) {
		JDBCEnvironment.enabledDatabase = enabledDatabase;
	}
}
