package org.phosphoresce.webcore.ext.poi;

import org.phosphoresce.webcore.core.exception.WebcoreConfigurationException;
import org.phosphoresce.webcore.core.logger.CodeConvertLogger;
import org.slf4j.LoggerFactory;

/**
 * 環境定義リソースに定義されている内容を元にExcel動的生成アドオンモジュールの初期化を行うインタフェースを提供します。<br>
 * <p>
 * 環境定義に当アドオンモジュールの初期化クラスが指定されている場合に{@link org.phosphoresce.webcore.core.GlobalConfiguration}より呼び出され、初期化処理が実行されます。<br>
 * クラス概要はコアモジュール側パッケージドキュメントを参照してください。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2011/12/24	Kitagawa		新規作成
 *-->
 */
public class PoiConfiguration implements PoiConstants {

	/** ロガーオブジェクト */
	private static CodeConvertLogger log = new CodeConvertLogger(LoggerFactory.getLogger(PoiConfiguration.class));

	/**
	 * コンストラクタ<br>
	 */
	private PoiConfiguration() {
		super();
	}

	/**
	 * Excel動的生成アドオンアドオンの初期化処理を行います。<br>
	 * 当初期化処理はアプリケーション共通初期化処理が完了されていることが前提となります。<br>
	 */
	public synchronized static void configure() {
		try {
			log.output("FPOI00001");

			/*
			 * 当アドオンモジュールは環境定義のみのロード処理とします
			 */

			log.output("FPOI00002");
		} catch (Throwable e) {
			throw new WebcoreConfigurationException("FPOI00004", e);
		}
	}

	/**
	 * Excel動的生成アドオンの破棄処理を実行します。<br>
	 * @throws Throwable 終了処理中に予期せぬエラーが発生した場合にスローされます 
	 */
	public synchronized static void destroy() throws Throwable {
		log.output("FPOI00003");
	}
}
