package org.phosphoresce.webcore.ext.struts.action;

import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;

import org.phosphoresce.webcore.core.transaction.TransactionUser;
import org.phosphoresce.webcore.ext.struts.annotation.StrutsForm;

/**
 * アドオンモジュール内部処理で利用されるダミーのフォームを提供します。<br>
 * <p>
 * サーブレットアクションモジュールが存在しないリクエストパス受領時のリクエストプロセッサーで返却されるダミーのフォームとなります。<br>
 * また、当フォームクラスは任意の画面JSPのモックアップ表示を実現する為に、Mapインタフェースが実装され、JSP側から任意のBeanアクセスを可能としています。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2010/07/19	Kitagawa		新規作成
 * 2013/08/01	Kitagawa		モックアップ画面表示を可能とする為に汎用的なアクセッサインタフェースを持つフォームとする(Mapインタフェースを実装)
 *-->
 */
@StrutsForm
public final class DummyForm extends AbstractForm<TransactionUser> implements Map<String, Object> {

	/**
	 * フォームオブジェクトをリセットします。<br>
	 * @param servlet サーブレットコンテナオブジェクト
	 * @param session ウィンドウセッションオブジェクト
	 * @see org.phosphoresce.webcore.ext.struts.action.AbstractForm#reset(org.phosphoresce.webcore.ext.struts.action.ServletContainer, org.phosphoresce.webcore.ext.struts.action.WindowSessionContainer)
	 */
	@Override
	public void reset(ServletContainer<TransactionUser> servlet, WindowSessionContainer<TransactionUser> session) {
	}

	/**
	 * フォームオブジェクトを初期化します。<br>
	 * @param servlet サーブレットコンテナオブジェクト
	 * @param session ウィンドウセッションオブジェクト
	 * @see org.phosphoresce.webcore.ext.struts.action.AbstractForm#init(org.phosphoresce.webcore.ext.struts.action.ServletContainer, org.phosphoresce.webcore.ext.struts.action.WindowSessionContainer)
	 */
	@Override
	public void init(ServletContainer<TransactionUser> servlet, WindowSessionContainer<TransactionUser> session) {
	}

	/**
	 * JSPモックアップ表示を可能とする為に実装されているメソッドですが、如何なる処理も実装されていません。<br>
	 * @return 常に0を返却します
	 * @see java.util.Map#size()
	 */
	@Override
	public final int size() {
		return 0;
	}

	/**
	 * JSPモックアップ表示を可能とする為に実装されているメソッドですが、如何なる処理も実装されていません。<br>
	 * @return 常にtrueを返却します
	 * @see java.util.Map#isEmpty()
	 */
	@Override
	public final boolean isEmpty() {
		return true;
	}

	/**
	 * JSPモックアップ表示を可能とする為に実装されているメソッドですが、如何なる処理も実装されていません。<br>
	 * @return 常にfalseを返却します
	 * @see java.util.Map#containsKey(java.lang.Object)
	 */
	@Override
	public final boolean containsKey(Object key) {
		return false;
	}

	/**
	 * JSPモックアップ表示を可能とする為に実装されているメソッドですが、如何なる処理も実装されていません。<br>
	 * @return 常にfalseを返却します
	 * @see java.util.Map#containsValue(java.lang.Object)
	 */
	@Override
	public final boolean containsValue(Object value) {
		return false;
	}

	/**
	 * JSPモックアップ表示を可能とする為に実装されているメソッドですが、如何なる処理も実装されていません。<br>
	 * @param key フォームプロパティ
	 * @return 常にnullを返却します
	 * @see java.util.Map#get(java.lang.Object)
	 */
	@Override
	public final Object get(Object key) {
		return null;
	}

	/**
	 * JSPモックアップ表示を可能とする為に実装されているメソッドですが、如何なる処理も実装されていません。<br>
	 * @param key フォームプロパティ
	 * @param value フォーム値
	 * @return 常にnullを返却します
	 * @see java.util.Map#put(java.lang.Object, java.lang.Object)
	 */
	@Override
	public final Object put(String key, Object value) {
		return null;
	}

	/**
	 * JSPモックアップ表示を可能とする為に実装されているメソッドですが、如何なる処理も実装されていません。<br>
	 * @param key フォームプロパティ
	 * @return 常にnullを返却します
	 * @see java.util.Map#remove(java.lang.Object)
	 */
	@Override
	public final Object remove(Object key) {
		return null;
	}

	/**
	 * JSPモックアップ表示を可能とする為に実装されているメソッドですが、如何なる処理も実装されていません。<br>
	 * @see java.util.Map#putAll(java.util.Map)
	 */
	@Override
	public final void putAll(Map<? extends String, ? extends Object> m) {
	}

	/**
	 * JSPモックアップ表示を可能とする為に実装されているメソッドですが、如何なる処理も実装されていません。<br>
	 * @see java.util.Map#clear()
	 */
	@Override
	public final void clear() {
	}

	/**
	 * JSPモックアップ表示を可能とする為に実装されているメソッドですが、如何なる処理も実装されていません。<br>
	 * @return 常に空のHashSetが返却されます
	 * @see java.util.Map#keySet()
	 */
	@Override
	public final Set<String> keySet() {
		return new HashSet<String>();
	}

	/**
	 * JSPモックアップ表示を可能とする為に実装されているメソッドですが、如何なる処理も実装されていません。<br>
	 * @return 常に空のLinkedListが返却されます
	 * @see java.util.Map#values()
	 */
	@Override
	public final Collection<Object> values() {
		return new LinkedList<Object>();
	}

	/**
	 * JSPモックアップ表示を可能とする為に実装されているメソッドですが、如何なる処理も実装されていません。<br>
	 * @return 常に空のHashSetが返却されます
	 * @see java.util.Map#entrySet()
	 */
	@Override
	public final Set<java.util.Map.Entry<String, Object>> entrySet() {
		return new HashSet<java.util.Map.Entry<String, Object>>();
	}
}
