package org.phosphoresce.webcore.ext.struts.action;

import java.io.Serializable;
import java.util.Date;

import org.phosphoresce.webcore.core.transaction.TransactionUser;

/**
 * Strutsアドオンモジュール内でのログインユーザの基本情報に関するインタフェースを提供します。<br>
 * <p>
 * このクラスが管理するログインユーザー情報はフレームワーク上で利用する最低限の基本情報のみとなります。<br>
 * アプリケーションに特化したユーザー情報は{@link #setLoginInfo(TransactionUser)}で設定することで、ログイン後に情報を持ちまわることが出来ます。<br>
 * 尚、ここに指定するユーザー情報は{@link TransactionUser}を実装したトランザクション処理を行うユーザークラスインスタンスとする必要があります。<br>
 * </p>
 * 
 * @param <L> アプリケーション実行ユーザーのトランザクション処理ユーザークラス
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2010/07/19	Kitagawa		新規作成
 * 2012/07/05	Kitagawa		全体的に再構築
 *-->
 */
public final class LoginUser<L extends TransactionUser> implements Serializable {

	/** ユーザコード */
	private String userCode;

	/** ログイン日時 */
	private Date loginDate;

	/** ログイン情報 */
	private L loginInfo;

	/**
	 * コンストラクタ<br>
	 */
	public LoginUser() {
		super();
	}

	/**
	 * ユーザコードを取得します。<br>
	 * @return ユーザコード
	 */
	public String getUserCode() {
		return userCode;
	}

	/**
	 * ユーザコードを設定します。<br>
	 * @param userCode ユーザコード
	 */
	public void setUserCode(String userCode) {
		this.userCode = userCode;
	}

	/**
	 * ログイン日時を取得します。<br>
	 * @return ログイン日時
	 */
	public Date getLoginDate() {
		return loginDate;
	}

	/**
	 * ログイン日時を設定します。<br>
	 * @param loginDate ログイン日時
	 */
	public void setLoginDate(Date loginDate) {
		this.loginDate = loginDate;
	}

	/**
	 * ログイン情報を取得します。<br>
	 * @return ログイン情報
	 */
	public L getLoginInfo() {
		return loginInfo;
	}

	/**
	 * ログイン情報を設定します。<br>
	 * @param loginInfo ログイン情報
	 */
	public void setLoginInfo(L loginInfo) {
		this.loginInfo = loginInfo;
	}

	/**
	 * クラス情報を文字列で取得します。<br>
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("{");
		builder.append("userCode=");
		builder.append(userCode);
		builder.append(", ");
		builder.append("loginDate=");
		builder.append(loginDate);
		builder.append(", ");
		builder.append("loginInfo=");
		builder.append(loginInfo);
		builder.append("}");
		return builder.toString();
	}
}
