package org.phosphoresce.webcore.ext.struts.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import org.phosphoresce.webcore.ext.struts.action.DummyTransactionManager;

/**
 * アクションクラスの動作挙動を定義するアノテーションです。<br>
 * <p>
 * 全てのアクションクラスはこのアノテーションを付与して実装します。<br>
 * アクションで利用するディフォルトトランザクションの定義や、名前付きトランザクション定義はこのアノテーションで定義します。<br>
 * </p>
 * <p>
 * 尚、システムで共通のトランザクション利用等の定義を行う場合は、システム共通の上位アクションクラスにアノテーションを定義して利用することで、
 * 各機能毎のアクションクラスにおける個別のアノテーションを実装する必要がなくなります。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2010/07/19	Kitagawa		新規作成
 * 2012/07/04	Kitagawa		全体的に再構築
 *-->
 */
@Inherited
@Retention(RetentionPolicy.RUNTIME)
@Target({ ElementType.TYPE })
@Documented
public @interface StrutsAction {

	/**
	 * ディフォルトのデータベーストランザクション管理クラスを指定します。<br>
	 * ここで指定したトランザクションが{@link StrutsMethod#transaction()}にtrueを指定した場合にオープンされて利用されます。<br>
	 */
	public abstract StrutsDefaultTransaction transaction() default @StrutsDefaultTransaction(transactionClass = DummyTransactionManager.class);

	/**
	 * 名前付きルックアップデータベーストランザクション管理クラスを複数指定します。<br>
	 * ここで指定したトランザクションが{@link StrutsMethod#namedTransactions()}で対応するルックアップ名を指定してオープンすることが可能となります。<br>
	 */
	public abstract StrutsTransaction[] namedTransactions() default {};
}
