package org.phosphoresce.webcore.ext.struts.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import org.phosphoresce.webcore.ext.struts.enums.CacheReflectType;

/**
 * アクションクラスメソッドの挙動定義を行うアノテーションです。<br>
 * <p>
 * アクションクラスのイベントメソッドに付与して利用するアノテーションで、イベント処理挙動定義を行います。<br>
 * アクションメソッドは必ずこのアノテーションが定義されている必要があります。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2010/07/19	Kitagawa		新規作成
 * 2012/07/04	Kitagawa		全体的に再構築
 * 2012/11/07	Kitagawa		編集済み状態クリアフラグ追加
 * 2012/12/07	Kitagawa		トランザクショントークンチェックはディフォルト動作に変更
 *-->
 */
@Inherited
@Retention(RetentionPolicy.RUNTIME)
@Target({ ElementType.METHOD })
@Documented
public @interface StrutsMethod {

	/**
	 * アクション実行時のフォームフィールドのキャッシュ反映方法を指定します。<br>
	 */
	public abstract CacheReflectType reflectCache() default CacheReflectType.MARKED_FIELD;

	/**
	 * アクション実行時に呼び出すフォームバリデーションメソッドを指定します。<br>
	 * 未指定の場合は、バリデーションメソッドは実行されません。<br>
	 */
	public abstract String validate() default "";

	/**
	 * バリデーションエラーが発生した際に遷移するパスを指定します。<br>
	 * 未指定の場合はアクション実行パスに対応したJSPに遷移します。<br>
	 */
	public abstract String input() default "";

	/**
	 * ロジック実行時にディフォルトトランザクションを利用する場合にtrueを指定します。<br>
	 * アクションクラスアノテーションにおいて{@link StrutsAction#transaction()}が指定されている必要があります。<br>
	 */
	public abstract boolean transaction() default false;

	/**
	 * 名前付きデータベーストランザクションを利用する際に利用するトランザクションルックアップ名を複数指定します。<br>
	 * アクションクラスアノテーションにおいて{@link StrutsAction#namedTransactions()}に対象のルックアップ名でトランザクション定義が行われている必要があります。<br>
	 */
	public abstract String[] namedTransactions() default {};

	/**
	 * アクション実行時にセッションタイムアウトチェックを行う場合にtrueを指定します。<br>
	 */
	public abstract boolean checkSesion() default true;

	/**
	 * アクション実行時にログイン状態チェックを行う場合にtrueを指定します。<br>
	 */
	public abstract boolean checkLogin() default true;

	/**
	 * アクション実行時に遷移トークンチェックを行う場合にtrueを指定します。<br>
	 */
	public abstract boolean checkToken() default false;

	/**
	 * アクション実行時に項目編集済みフラグをクリアする場合にtrueを指定します。<br>
	 * 登録処理などで編集状態をクリアする際に指定されることが想定されます。<br>
	 */
	public abstract boolean clearEdited() default false;

	/**
	 * アクション実行時に画面スクロール位置をクリアする場合にtrueを指定します。<br>
	 * 登録処理や検索処理などで画面表示位置を初期化する場合などで指定されることが想定されます。<br>
	 */
	public abstract boolean clearWindowScroll() default false;
}
