package org.phosphoresce.webcore.ext.struts.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import org.phosphoresce.webcore.core.transaction.TransactionManager;

/**
 * アクションで利用するトランザクションを定義するアノテーションです。<br>
 * <p>
 * アクションから利用されるロジックにおいてトランザクション利用を行う際に利用されるトランザクションを定義する為のアノテーションです。<br>
 * 複数のトランザクションを利用する場合に、名前付きのトランザクションとして{@link StrutsAction#namedTransactions()}に複数定義して利用します。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2010/07/19	Kitagawa		新規作成
 * 2012/07/04	Kitagawa		全体的に再構築
 *-->
 */
@Inherited
@Retention(RetentionPolicy.RUNTIME)
@Target({ ElementType.METHOD })
@Documented
public @interface StrutsTransaction {

	/**
	 * ロジッククラスにおいて利用する際にルックアップする名前を指定します。<br>
	 */
	public abstract String lookup();

	/**
	 * トランザクション管理として利用するトランザクション管理クラスを指定します。<br>
	 */
	public abstract Class<? extends TransactionManager> transactionClass();

	/**
	 * トランザクション接続を行うための環境定義において名前空間指定がある場合にその名前空間を指定します。<br>
	 * ディフォルト定義(名前空間定義なし)を利用する場合は指定する必要はありません。<br>
	 */
	public abstract String namespace() default "";
}
