package org.phosphoresce.webcore.ext.struts.exception;

import org.phosphoresce.webcore.core.config.Config;
import org.phosphoresce.webcore.core.transaction.TransactionUser;
import org.phosphoresce.webcore.ext.struts.StrutsConfigNames;
import org.phosphoresce.webcore.ext.struts.action.AbstractForm;
import org.phosphoresce.webcore.ext.struts.action.ActionContext;
import org.phosphoresce.webcore.ext.struts.action.Forward;
import org.phosphoresce.webcore.ext.struts.action.MessageSet;
import org.phosphoresce.webcore.ext.struts.util.StrutsUtil;

/**
 * フォームバリデーションエラー発生時の例外クラスです。<br>
 * <p>
 * アクション実行時に実行されたバリデーションチェックでエラーとなった場合にスローされる例外です。<br>
 * この例外クラスは{@link StrutsModuleApplicationException}が継承された例外クラスであり、アプリケーションエラーとして扱われます。<br>
 * </p>
 * <p>
 * 画面上に設定されるエラーメッセージはバリデーションチェックにおいて設定されたメッセージとし、当該例外クラスからの個別メッセージ設定は行われません。<br>
 * また、遷移先はアクションが実行されたJSPとなります。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2010/07/19	Kitagawa		新規作成
 *-->
 */
public class StrutsFormValidationException extends StrutsModuleApplicationException {

	/**
	 * コンストラクタ<br>
	 */
	public StrutsFormValidationException() {
		super();
	}

	/**
	 * アプリケーションメッセージを生成します。<br>
	 * @param actionContext アクションコンテキストオブジェクト
	 * @return アプリケーションメッセージ
	 * @see org.phosphoresce.webcore.ext.struts.exception.StrutsModuleApplicationException#createApplicationMessage(org.phosphoresce.webcore.ext.struts.action.ActionContext)
	 */
	@Override
	public <L extends TransactionUser, F extends AbstractForm<L>> MessageSet createApplicationMessage(ActionContext<L, F> actionContext) {
		MessageSet set = null;
		/*
		 * バリデーションエラーはバリデーションメソッドにて設定されたメッセージのみとする
		 */
		return set;
	}

	/**
	 * 遷移用アクションフォワードを生成します。<br>
	 * @param actionContext アクションコンテキストオブジェクト
	 * @return 遷移用アクションフォワード
	 * @see org.phosphoresce.webcore.ext.struts.exception.StrutsModuleApplicationException#createActionForward(org.phosphoresce.webcore.ext.struts.action.ActionContext)
	 */
	@Override
	public <L extends TransactionUser, F extends AbstractForm<L>> Forward createActionForward(ActionContext<L, F> actionContext) {
		Forward forward = null;
		if (actionContext.isJSONReuqest()) {
			forward = null;
		} else if (actionContext.isPopupRequest()) {
			String actionPath = StrutsUtil.getRequestActionPath(actionContext.getServletRequest(), actionContext.getModuleConfig());
			String jspName = StrutsUtil.getRequestFunction(actionPath) + ".jsp";
			forward = new Forward();
			forward.setPath(jspName);
			forward.setRedirect(false);
		} else {
			String actionPath = StrutsUtil.getRequestActionPath(actionContext.getServletRequest(), actionContext.getModuleConfig());
			String jspName = StrutsUtil.getRequestFunction(actionPath) + ".jsp";
			forward = new Forward();
			forward.setPath(jspName);
			forward.setRedirect(false);
		}
		return forward;
	}

	/**
	 * レスポンスステータスを生成します。<br>
	 * @param actionContext アクションコンテキストオブジェクト
	 * @return レスポンスステータス(特別なステータスコードが不要であればnullを返却します)
	 * @see org.phosphoresce.webcore.ext.struts.exception.StrutsModuleApplicationException#createResponseStatus(org.phosphoresce.webcore.ext.struts.action.ActionContext)
	 */
	@Override
	public <L extends TransactionUser, F extends AbstractForm<L>> Integer createResponseStatus(ActionContext<L, F> actionContext) {
		if (Config.isEmpty(StrutsConfigNames.STRUTS_STATUS_VALIDATIONERROR)) {
			return null;
		} else {
			return Config.getInteger(StrutsConfigNames.STRUTS_STATUS_VALIDATIONERROR);
		}
	}
}
